angular.module('beamng.apps')

.directive('fusionForcedInduction', [function() {
  // noinspection CssUnknownTarget
  return {
    template:
        '<div>' +
          '<link rel="stylesheet" href="../../modules/apps/FusionForcedInduction/app.css">' +
          '<link rel="stylesheet" href="../../tachometers/base.css">' +
          // Pre-load font
          '<canvas class="canvas" id="FFI_bgCanvas" width="150" height="150"></canvas>' +
          '<canvas class="canvas" id="FFI_canvas"   width="150" height="150"></canvas>' +
        '</div>',
    replace: true,
    restrict: 'EA',
    link: function ($scope, $element) {
      const ID = "FFI";
      const REQ_STREAMS = ["forcedInductionInfo"];
      const REQ_SERVICES = [
        "base/apps", "base/base", "base/colours", "base/gauge", "base/shapes",
        "fusionService"
      ];

      StreamsManager.add(REQ_STREAMS);


      // Below has to be in all apps, and cannot be added to the BaseApp class since the file that
      // is in hasn't been loaded until this code has run

      let scriptsLoaded = 0;
      // Add in the script files for the base service
      for (let service of REQ_SERVICES) {
        let script = document.createElement("script");
        script.src = "../../tachometers/services/" + service + ".js";
        // Add in this directives ID to avoid conflicts between all of these modded apps
        script.id = service + ID;
        $element.append(script);

        script.onload = function() {
          scriptsLoaded++;
          // Wait for all scripts to load
          if (scriptsLoaded === REQ_SERVICES.length)
            onload();
        };
      }


      let app = null;
      let TachoApps = null;

      //////////////////////////////
      // Rendering/Main Functions //
      //////////////////////////////

      /**
       * Returns a list of all dynamic variables that will/may change between frames for the vehicle
       * @param {Object[]} forcedInductionInfo - the forced induction information stream
       */
      function getDynamicVars(forcedInductionInfo) {
        return {
          boost: forcedInductionInfo.boost
        };
      }


      $scope.$on("streamsUpdate", function (event, streams) {
        let forcedInductionInfo = streams.forcedInductionInfo;

        // Wait until ready
        if (!app || app.state === app.states.preInit || !forcedInductionInfo)
          return;

        let baseVars = getDynamicVars(forcedInductionInfo);

        // Perform first time setup if necessary
        if (app.state === app.states.init) {
          app.initFields(baseVars.boost, forcedInductionInfo.maxBoost);
          app.load(bngApi);
          app.drawBackgroundCanvas();
          app.state = app.states.loaded;
        }

        app.setCurrBoost(baseVars.boost);
        app.draw();
      });

      $scope.$on("$destroy", () => {StreamsManager.remove(REQ_STREAMS)});
      $scope.$on("VehicleFocusChanged", () => {if (TachoApps && app) TachoApps.BaseApp.reload(app, true)});
      $scope.$on("VehicleChange", () => {if (TachoApps && app) TachoApps.BaseApp.reload(app, false)});

      /**
       * Loads the external services and initialises some base constants.
       */
      function onload() {
        // Request and setup the services
        let injector = angular.injector(["beamng.apps"]);
        let ModFusionBase = injector.get("ModFusionBase");
        TachoApps = injector.get("TachoApps");

        app = new ModFusionBase.FusionBoostApp(ID, document, bngApi, UiUnits);
      }
    }
  };
}]);