angular.module('beamng.apps')
.directive('velocityTacho', [function() {
  // noinspection CssUnknownTarget
  return {
    template:
        '<div id="VT_container">' +
          '<link rel="stylesheet" href="../../modules/apps/VelocityTacho/app.css">' +
          '<link rel="stylesheet" href="../../tachometers/base.css">' +
          '<canvas class="canvas" id="VT_bgCanvas" width="300" height="300"></canvas>' +
          '<canvas class="canvas" id="VT_canvas"   width="300" height="300"></canvas>' +
          // '<div ng-include="\'../ui/tachometers/colours.html\'"></div>' +
        '</div>',
    replace: true,
    restrict: 'EA',
    link: function ($scope, $element) {
      // The internal ID for this app
      const ID = "VT";
      const REQ_STREAMS = ["electrics", "engineInfo"];
      const REQ_SERVICES = [
          "base/apps", "base/base", "base/colours", "base/gauge", "base/shapes",
          "velocityService"
      ];

      StreamsManager.add(REQ_STREAMS);


      let scriptsLoaded = 0;
      // Add in the script files for the base service
      for (let service of REQ_SERVICES) {
        let script = document.createElement("script");
        script.src = "../../tachometers/services/" + service + ".js";
        // Add in this directives ID to avoid conflicts between all of these modded apps
        script.id = service + ID;
        $element.append(script);

        script.onload = function() {
          scriptsLoaded++;
          // Wait for all scripts to load
          if (scriptsLoaded === REQ_SERVICES.length)
            onload();
        };
      }


      let app = null;
      let TachoApps = null;

      //////////////////////////////
      // Rendering/Main Functions //
      //////////////////////////////

      $scope.$on("streamsUpdate", (event, streams) => {
        if (TachoApps)
          TachoApps.TachometerApp.sendStreamEvent(streams, app, bngApi);
      });

      $scope.$on("$destroy", () => {StreamsManager.remove(REQ_STREAMS)});
      $scope.$on("VehicleFocusChanged", () => {if (TachoApps && app) TachoApps.BaseApp.reload(app, true)});
      $scope.$on("VehicleChange", () => {if (TachoApps && app) TachoApps.BaseApp.reload(app, false)});

      // $scope.$on('app:resized', function(event, data) {
      //   // TachoBase.TachometerApp.changeSize(app, data.width, data.height);
      // });

      /**
       * Loads the external services and initialises some base constants.
       */
      function onload() {
        // Request and setup the services
        let injector = angular.injector(["beamng.apps"]);
        let ModBase = injector.get("ModVelocityBase");
        TachoApps = injector.get("TachoApps");

        app = new ModBase.VelocityTachoApp(ID, document, bngApi, UiUnits);
      }
    }
  };
}]);